// ******************************************************************************* 
// *********
// *********      DEVICE DESCRIPTION
// *********      Target:       HCF / AMS / FC375 / PDM
// *********      DD Bus Typ:   HART
// *********      Date:         15.01.2013
// *********      Device Name:  PFMH
// *********      Label String: V1.0.0.1
// *********      Built Number: 1
// *********
// ******************************************************************************* 
//                                                                                 
// Manufactor:    Baumer                                      
//                
                                                   
//                                                               
//                Tel:
//                                                   
// Copyright:      ifak system GmbH, All Rights Reserved              
// Department:    Automation / Aut                                   
// Author:        fbi / mhu                                   
//                                                                                 
// *******************************************************************************

/*
**********************************************************************
** Manufacturer, Device type, Device revision, DD-revision
**********************************************************************
*/
MANUFACTURER 	BAUMER,
DEVICE_TYPE 	_PFMH, 

DEVICE_REVISION 1,
DD_REVISION 	1

/*
**********************************************************************
** Locking Defines
**********************************************************************
*/

#define DEFAULT_ADMIN_PASSWORD 8888
#define DEFAULT_OPERATOR_PASSWORD 1921
#define DEFAULT_MASTER_PASSWORD 7378

#define OPERATING_MODE_VALIDITY IF ((User_mode >= 1) || (PW_protection_enable == 0)) { TRUE; } ELSE { FALSE; }
#define ADMIN_MODE_VALIDITY IF ((User_mode == 2) || (PW_protection_enable == 0)) { TRUE; } ELSE { FALSE; }
#define OPERATING_MODE_HANDLING IF ((User_mode >= 1) || (PW_protection_enable == 0)) { READ & WRITE; } ELSE { READ; }
#define ADMIN_MODE_HANDLING IF ((User_mode == 2) || (PW_protection_enable == 0)) { READ & WRITE; } ELSE { READ; }

#define CYCLE_TIME_PRESS_CHART 				5000 // 0,5s update time
#define CYCLE_TIME_TEMP_CHART 				5000 // 3s update time
#define CYCLE_TIME_CURRENT_CHART			5000 // 3s update time
#define CYCLE_TIME_PRESS_CHART_2 			1000 // 0,5s update time
#define CYCLE_TIME_TEMP_CHART_2 			1000 // 3s update time
#define CYCLE_TIME_CURRENT_CHART_2			1000 // 3s update time

#define WITH_CONSTANT_UNIT_DEPEND

#ifdef AMS_APPLICATION
	#undef AMS_APPLICATION
#endif

#define MIN_VALUE_pv_value_desired	(pv_value - (pv_upper_sensor_limit - pv_lower_sensor_limit) / 10 )
#define MAX_VALUE_pv_value_desired	(pv_value + (pv_upper_sensor_limit - pv_lower_sensor_limit) / 10 )
//#define MIN_VALUE_pv_value_desired	(pv_value - (upperRange_value_2 - lowerRange_value_2) / 10 )
//#define MAX_VALUE_pv_value_desired	(pv_value + (upperRange_value_2 - lowerRange_value_2) / 10 )

#ifdef _EDDY_
VARIABLE time_stamp 
{    
    LABEL "Time Stamp";
    HELP "Relative time value on network";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE TIME_VALUE;
}
#endif

/*
**********************************************************************
** Standard Include-files
**********************************************************************
*/

#include "macros.h"
#include "methods.h"

/*
**********************************************************************
** Device specific Include-files
**********************************************************************
*/

#include "60F00101_dic.dd"
#include "60F00101_var.dd"

/*
**********************************************************************
** HART Smart Communications Protocol - Common Tables
** Source Filename: Tabl20_4
**********************************************************************
*/

IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4
//fbi	IMPORT "Tables_20_4.ddl"
{
	HART_DEVICE_TABLES;  // Pulls in all the items for typical HART devices.
 
   REDEFINITIONS
   {
	   
        REDEFINE VARIABLE device_type
        {
    	    LABEL device_type_lbl;
    	    HELP device_type_lbl;
           TYPE ENUMERATED (2)
            {
		    
                { 0x60F0,  "CombiPress" }
            }
        }

        VARIABLE manufacturer_id
        {
            // The type must be redefined if the Manufacturer code is not listed
            //  in the Common Tables DD
            //
	    REDEFINE LABEL manufacturer_id_lbl;
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your Manufacturer code and appropriate label
                { 0x0060,  "BAUMER" } 
            }
       }

        VARIABLE private_label_distributor
        {
            //
            //  The type must be redefined if the Private Label code is not listed
            //  in the Common Tables DD
            // 
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your manufactures id and appropriate label
                { 0x0060,  "BAUMER" } 
            }
        }
    	VARIABLE extended_fld_device_status     // see table 17
    	{
    		REDEFINE LABEL ex_device_status_lbl;
/*    		TYPE BIT_ENUMERATED
    		{     
    //			{ 0x00, no_error_lbl },
    			{ 0x01, maintenance_required_lbl },
    			{ 0x02, "Out of specification" } 
    		}
    */
    	}
        REDEFINE VARIABLE loop_current_mode_codes
        {
		LABEL loop_current_lbl;
		TYPE ENUMERATED 
		{
			{ 0, disabled_loop_current_mode_lbl },  
			{ 1, enabled_loop_current_mode_lbl }
            #ifdef _PDM_ 
			DEFAULT_VALUE 1;
            #endif
            #ifdef  _EDDY_
			DEFAULT_VALUE 1;
            #endif
		}
        }
    	REDEFINE VARIABLE loop_current_mode
    	{
    	    LABEL loop_current_lbl;
    	    TYPE ENUMERATED 
    	    {
    			{ 0, disabled_loop_current_mode_lbl },  
    			{ 1, enabled_loop_current_mode_lbl }
    	    }
    	}
    	REDEFINE VARIABLE device_status
    	{
    	    LABEL [field_device_status];
    	    HELP [field_device_status_help];
    	    CLASS HART & DYNAMIC & DIAGNOSTIC;
    	    HANDLING READ;
    	    TYPE BIT_ENUMERATED
    	    {
    		{ 0x01, [PV_out_of_limits],             DV1 (AUTO & BAD) },
    		{ 0x02, [Non_PV_out_of_limits],         DV2 (AUTO & BAD) },
    		{ 0x04, "PV Analog Channel Saturated",  AO1 (AUTO & BAD) },
    		{ 0x08, "PV Analog Channel Fixed",      AO1 (MANUAL & GOOD) },
    		{ 0x10, [more_status_available],        MORE },
    		{ 0x20, [cold_start],                   MISC },
    		{ 0x40, [configuration_changed],        MISC },
    		{ 0x80, [device_malfunction],           MORE}
    	    }
    		#ifdef _PDM_
    		STYLE "MULTILINE 70 8" ;
    		#endif
    	}
    	REDEFINE VARIABLE write_protect
    	{
    	    LABEL [write_protect];
    	    HELP [write_protect_mode_help];
    	    CLASS DEVICE;
    	    HANDLING READ;
    	    TYPE ENUMERATED
    	    {
    		WRITE_PROTECT_CODE(0),  /* Not Write Protected */
    		WRITE_PROTECT_CODE(1),  /* Write Protected */
    		WRITE_PROTECT_CODE(251) /* None */
    	    }
    	}
        VARIABLE transfer_function_code
        {
            REDEFINE TYPE ENUMERATED
            {
        
                { 0,   linear_enum,     [linear_help] },
                { 4,   special_curve_enum,[special_curve_help] }
            }
        }
    
    }
}
/*
**********************************************************************
** HART Universal Commands
** Source Filename: univ7.ddl
**********************************************************************
*/

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
//fbi	IMPORT "Universal_7_2.ddl"
{
    // All HART devices must support all Universal Commands
    EVERYTHING;

    REDEFINITIONS
    {
	DELETE COMMAND read_device_variables_and_status;		//	    NUMBER 9;;
	    
      // This must be redefined if the Manufacturer is not listed in the Common Tables DD
      VARIABLE polling_address
      {
         REDEFINE TYPE UNSIGNED_INTEGER
         {
            MIN_VALUE 0;
            MAX_VALUE 63;
         }
      }
      
      VARIABLE tag
      {
         REDEFINE LABEL tag_lbl;
      }

      VARIABLE longTag
      {
         REDEFINE LABEL long_tag_lbl;
      }

      
	VARIABLE polling_address
	{
	    REDEFINE LABEL polling_address_lbl;
	}

	VARIABLE software_revision
	{
	    REDEFINE LABEL software_revision_lbl;
	}
	
	VARIABLE hardware_revision
	{
	    REDEFINE LABEL hardware_revision_lbl;
	}

	VARIABLE device_id
	{
		REDEFINE LABEL device_identification_lbl;
	}
	
	COMMAND read_unique_identifier 		// NUMBER 0;
	{
	    REDEFINE TRANSACTION
	    {
		REQUEST {}
		REPLY
		{
			    // every variable of cmd 0 must be listed separate. PDM won't use the define IDENTIFY_REPLY_7 from Macros.h so it can't read these variables
			    //XXX so ist der eigentliche response... aber PDM SP2 verwendet noch 1Byte Manufacturer & Device ID

			    response_code, device_status, 
			    254, 
			    device_type, 
			    request_preambles, 
			    universal_revision, 
			    transmitter_revision, 
			    software_revision, 
			    hardware_revision <0xf8>, 
			    physical_signaling_code <0x07>,
			    device_flags, 
			    device_id, 
			    response_preambles, 
			    max_num_device_variables,
			    config_change_counter,
			    extended_fld_device_status, 
			    manufacturer_id,
			    private_label_distributor, 
			    device_profile
		}
	    }
	}
	/*
	COMMAND read_pv 		//NUMBER 1;
	{
	    REDEFINE TRANSACTION
	    {
		REQUEST {}
		REPLY
		{
		    response_code, device_status, 
		   // PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE
			0x00, 0x00, 0x00, 0x00, 0x00
		}
	    }
	}
*/

      // The transaction will need to be redefine to show the correct
      // number of dynamic variables returned in this command
      // (Command 3)      
        COMMAND read_dynamic_variables_and_pv_current
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DAQ.ANALOG_VALUE,
                    PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE
                }
            }
        }
        

        /* 
         * The transaction will need to be redefined to show the correct
         * number of dynamic variables returned in this command 
         * (Command 8)
         */
        COMMAND read_dynamic_variable_classification		//	    NUMBER 8;
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DEVICE_VARIABLE.CLASSIFICATION,
                    SV.DEVICE_VARIABLE.CLASSIFICATION
                }
            }
        }

	COMMAND read_pv_sensor_info			 // NUMBER 14;
	{
	    REDEFINE TRANSACTION
	    {
		REQUEST {}
		REPLY
		{
		    response_code, device_status,
		    PV.DEVICE_VARIABLE.SENSOR_SERIAL_NUMBER, 
		    PV.DEVICE_VARIABLE.DIGITAL_UNITS, 
                pv_upper_sensor_limit,
                pv_lower_sensor_limit
		    //PV.DEVICE_VARIABLE.UPPER_SENSOR_LIMIT,
		    //PV.DEVICE_VARIABLE.LOWER_SENSOR_LIMIT,
		    PV.DEVICE_VARIABLE.MINIMUM_SPAN
		}
	    }
	}

	COMMAND read_pv_output_info		//NUMBER 15;
	{
	    REDEFINE TRANSACTION
	    {
    		REQUEST {}
    		REPLY
            {
                response_code, device_status,
                PV.DAQ.ALARM_CODE, 
                PV.RANGING.TRANSFER_FUNCTION,
                pv_unit, 	///PV.RANGING.RANGE_UNITS, 
                upperRange_value_2,
                //PV.RANGING.UPPER_RANGE_VALUE,
                lowerRange_value_2,
                //PV.RANGING.LOWER_RANGE_VALUE,
                PV.DEVICE_VARIABLE.DAMPING_VALUE,
                write_protect, 
                250,
                PV.DAQ.ANALOG_CHANNEL_FLAGS
            }
	    }
	}

	COMMAND write_long_tag		//	    NUMBER 22;
	{
	    REDEFINE TRANSACTION
	    {
		REQUEST
		{
		    longTag
		}
		REPLY
		{
		    response_code, device_status, longTag
		}
	    }
	}
	

        // Remove the command 48 status bytes that are not supported by the device
        // See also command 48 related items in the common practice imports.

        COMMAND read_additional_device_status   //NUMBER 48;
        {
            REDEFINE TRANSACTION 0
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status, 
                    0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00,

                    extended_fld_device_status, 0x00,

                    0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00
                }
            }

            REDEFINE TRANSACTION 1
            {
                REQUEST
                {
                    0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00,

                    extended_fld_device_status, 0x00,

                    0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00
                }
                REPLY
                {
                    response_code, device_status, 
                    0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00,

                    extended_fld_device_status, 0x00,

                    0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00
                }
            }
        }
    /*
        COMMAND read_additional_device_status
        {
            REDEFINE TRANSACTION 0
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,

                    extended_fld_device_status, 0x00,

                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3, analog_channel_fixed1
                }
            }

            REDEFINE TRANSACTION 1
            {
                REQUEST
                {
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO), analog_channel_fixed1 (INFO)
                }
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO), analog_channel_fixed1 (INFO)
                }
            }
        }
        */
    }
}

/*
**********************************************************************
** HART Standard Collections
** Source Filename: PV1_1.ddl
**********************************************************************
*/
IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
//fbi	IMPORT "pv1_1.ddl"
{
    EVERYTHING;
    REDEFINITIONS
    {
        /* 
         * Most devices use the same data item for range units (command 15) and the sensor units.
         * when this is true the following should be added.  if your device uses a different variable
         * for range units you should reference it here 
         */

        DELETE COLLECTION tertiary_variable;
        DELETE COLLECTION quaternary_variable;
    
	REDEFINE VARIABLE transfer_function
	{
	    LABEL transfer_function_lbl;
	    HELP transfer_function_lbl;
	    HANDLING READ & WRITE;
	    TYPE ENUMERATED
	    {
		TRANSFER_FUNCTION_CODE (0),     // Linear 
		//TRANSFER_FUNCTION_CODE (1),     // Square Root 
		TRANSFER_FUNCTION_CODE (4)      // Spezial
	    }
	#ifdef _PDM_
		DEFAULT_VALUE 4;
	#endif
	}

        ARRAY OF COLLECTION dynamic_variables
        {
            REDEFINE ELEMENTS
            {
                PRIMARY,        primary_variable,       "";
                SECONDARY,      secondary_variable,     "";
            }
        }
        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {
                ADD RANGE_UNITS, deviceVariables[0].DIGITAL_UNITS, [range_units], [range_units_help];
 		REDEFINE UPPER_RANGE_VALUE,      upperRange_value_2,   pv_urv_lbl,                  [upper_range_value_help];
		REDEFINE LOWER_RANGE_VALUE,      lowerRange_value_2,   pv_lrv_lbl,                  [lower_range_value_help];
           } 
        }
        COLLECTION OF VARIABLE analog_io
        {
            MEMBERS
            {
                /* Analog Information */
                REDEFINE ANALOG_VALUE,           loopCurrent,        loop_current_lbl,         [analog_value_help];
		REDEFINE ANALOG_CHANNEL_FLAGS,   loop_flags,         [analog_channel_flags], [analog_channel_flags_help];
            }
        }
   }
}
/*
**********************************************************************
** HART Common Practice
** Source Filename: Common9.ddl
**********************************************************************
*/

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
//fbi	IMPORT "Common_9.ddl"
{
	
    COMMAND write_pv_damping_value;                      // Command 34
    COMMAND write_pv_range_values;                      // Command 35
    COMMAND enter_exit_fixed_pv_current_mode;           // Command 40
    COMMAND set_pv_zero;                   		// Command 43
    COMMAND trim_pv_current_dac_zero;                   // Command 45
    COMMAND trim_pv_current_dac_gain;                   // Command 46
    COMMAND write_pv_transfer_function;			// Command 47
    COMMAND write_number_of_response_preambles;         // Command 59
    COMMAND squawk;                                     // Command 72
    COMMAND read_device_variable_trim_point;		// Command 80
    COMMAND write_device_variable_trim_point;           // Command 82
    COMMAND reset_device_variable_trim;           	// Command 83
	

    METHOD return_to_normal;
    METHOD warning_message;
    METHOD device_self_test;
    METHOD device_master_reset;
    METHOD transmitter_loop_test;
    METHOD leave_fixed_current_mode;
    METHOD transmitter_dac_trim;
    METHOD makeItSquawk;
    METHOD trim_warning;
    METHOD device_variable_trim_reset;

    VARIABLE device_variable_code;
    VARIABLE device_variable_trim_code;

    REDEFINITIONS
    {
        VARIABLE device_variable_trim_code
        {
            REDEFINE TYPE INDEX trimDeviceVariables; 
        }
    	COMMAND write_pv_range_values		//    NUMBER 35;
    	{
    	    REDEFINE TRANSACTION 
    	    {
        		REQUEST
        		{
        			PV.RANGING.RANGE_UNITS (INFO), 
        			upperRange_value_2,
        			//PV.RANGING.UPPER_RANGE_VALUE,
        			lowerRange_value_2
        			//PV.RANGING.LOWER_RANGE_VALUE
        		}
        		REPLY
        		{
        			response_code, device_status,
        			PV.RANGING.RANGE_UNITS (INFO), 
        			upperRange_value_2,
        			//PV.RANGING.UPPER_RANGE_VALUE,
        			lowerRange_value_2
        			//PV.RANGING.LOWER_RANGE_VALUE
        		}
    	    }
    	}
	COMMAND read_device_variable_trim_point		//    NUMBER 80;
	{
	    REDEFINE TRANSACTION
	    {
		REQUEST
		{
		    device_variable_trim_code (INFO,INDEX)
		}
		REPLY
		{
		    response_code, device_status,
		    device_variable_trim_code (INFO,INDEX),
		    deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
		    deviceVariables[device_variable_trim_code].LOWER_TRIM_POINT,
		    deviceVariables[device_variable_trim_code].UPPER_TRIM_POINT
		}
	    }
	}

	COMMAND write_device_variable_trim_point		// NUMBER 82;
	{
	    REDEFINE TRANSACTION
	    {
		REQUEST
		{
		    device_variable_trim_code (INFO,INDEX)
		    deviceVariables[device_variable_trim_code].TRIM_POINT_SUPPORT (INFO),
		    pv_unit,	//deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
		    pv_value_desired	//deviceVariables[device_variable_trim_code].DIGITAL_VALUE
		}
		REPLY
		{
		    response_code, device_status,
		    device_variable_trim_code (INFO,INDEX),
		    deviceVariables[device_variable_trim_code].TRIM_POINT_SUPPORT (INFO),
		    pv_unit,	//deviceVariables[device_variable_trim_code].TRIM_POINT_UNITS,
		    pv_value_desired	//deviceVariables[device_variable_trim_code].DIGITAL_VALUE
		}
	    }
	}
	METHOD device_variable_trim_reset
	{
		REDEFINE LABEL device_variable_trim_reset_lbl;
	}
     }
}

/*
 * Dynamic Variables Configuration
 */
#include "60F00101_PV.dd"
#include "60F00101_SV.dd"
#include "60F00101_unitrelation.dd"

ARRAY OF COLLECTION deviceVariables 
{
    ELEMENTS
    {
        0, pv_variable_collection, "";
        1, sv_variable_collection, "";
    }
}

ARRAY OF COLLECTION trimDeviceVariables 
{
    ELEMENTS
    {
        0, pv_variable_collection, "Pressure";
    }
}

/*
***************************************************************************
** below is list of menu declaritions
** they where take directly from menu.ddl an interoperable file
** this is a recommended display format derived by the HART communicator
** developers in conjunction with slave developers and end users.
*/

#include "60F00101_menu.dd"
#include "60F00101_mth.dd"
#include "60F00101_cmd.dd"
#include "60F00101_graph.dd"
#include "60F00101_tab.dd"


